from setuptools import setup, find_packages

setup(
    name='ffmpy',
    version='0.0.1',
    description='A Python wrapper around ffmpeg/ffprobe',
    author='Andriy Yurchuk',
    author_email='ay@mntw.re',
    url='https://github.com/Ch00k/ffmpy',
    packages=find_packages(),
)
